@@ -12,4 +12,7 @@ from . import pricefx
 from . import test
 from . import ssh
 from . import sftp
-from . import mongodb
\ No newline at end of file
+from . import mongodb
+from . import salesforce
+from . import recipe
+from . import s3
\ No newline at end of file
