@@ -7,10 +7,13 @@ from . import data as _data
 from . import batching as _batching
 from typing import Union as _Union
 
+from .format import tokenize
+
 
 def address(input: _Union[str, list], dataType: str) -> list:
     """
     Extract geographical information from unstructured text such as streets, cities or countries.
+    Requires WrangleWorks Account.
 
     e.g. '1100 Congress Ave, Austin, TX 78701, United States' -> '1100 Congress Ave'
 
@@ -34,9 +37,10 @@ def address(input: _Union[str, list], dataType: str) -> list:
     return results
 
     
-def attributes(input: _Union[str, list], responseContent: str = 'span', type: str = None) -> _Union[dict, list]:
+def attributes(input: _Union[str, list], responseContent: str = 'span', type: str = None, desiredUnit: str = None, bound: str = 'mid') -> _Union[dict, list]:
     """
     Extract numeric attributes from unstructured text such as lengths or voltages.
+    Requires WrangleWorks Account.
 
     >>> wrangles.extract.attributes('tape 25m')
     {'length': ['25m']}
@@ -44,6 +48,7 @@ def attributes(input: _Union[str, list], responseContent: str = 'span', type: st
     :param input: Input string or list of strings to be searched for attributes
     :param responseContent: (Optional, default Span) 'span' or 'object'. If span, returns original text, if object returns an object of value and dimension.
     :param type: (Optional) Specify which types of attributes to find. If omitted, a dict of all attributes types is returned
+    :param bound: (Optional, default mid). When returning an object, if the input is a range. e.g. 10-20mm, set the value to return. min, mid or max.
     """
     if isinstance(input, str): 
         json_data = [input]
@@ -52,8 +57,14 @@ def attributes(input: _Union[str, list], responseContent: str = 'span', type: st
 
     url = f'{_config.api_host}/wrangles/extract/attributes'
     params = {'responseFormat':'array', 'responseContent': responseContent}
-    if type is not None: params['attributeType'] = type
-
+    if type: params['attributeType'] = type
+    if desiredUnit: params['desiredUnit'] = desiredUnit
+    
+    if bound in ['min', 'mid', 'max']:
+        params['bound'] = bound
+    else:
+        raise ValueError('Invalid boundary setting. min, mid or max permitted.')
+    
     batch_size = 1000
 
     results = _batching.batch_api_calls(url, params, json_data, batch_size)
@@ -66,6 +77,7 @@ def attributes(input: _Union[str, list], responseContent: str = 'span', type: st
 def codes(input: _Union[str, list]) -> list:
     """
     Extract alphanumeric codes from unstructured text.
+    Requires WrangleWorks Account.
 
     e.g. 'Something ABC123ZZ something' -> 'ABC123ZZ'
 
@@ -89,6 +101,7 @@ def codes(input: _Union[str, list]) -> list:
 def custom(input: _Union[str, list], model_id: str) -> list:
     """
     Extract entities using a custom model.
+    Requires WrangleWorks Account and Subscription.
 
     :param input: A string or list of strings to searched for information.
     :param model_id: The model to be used to search for information.
@@ -101,9 +114,9 @@ def custom(input: _Union[str, list], model_id: str) -> list:
     else:
         raise TypeError('Invalid input data provided. The input must be either a string or a list of strings.')
         
-    # If the Model Id is not appropriate, raise error (Maybe more specific?)
-    if isinstance(model_id, dict) or len(model_id.split('-')) != 3:
-        raise ValueError('Incorrect model_id. May be missing "${ }" around value')
+    # If the Model Id is not appropriate, raise error (Only for Recipes)
+    if isinstance(model_id, dict):
+        raise ValueError('Incorrect model_id type.\nIf using Recipe, may be missing "${ }" around value')
         
     # Checking to see if GUID format is correct
     if [len(x) for x in model_id.split('-')] != [8, 4, 4]:
@@ -131,6 +144,7 @@ def custom(input: _Union[str, list], model_id: str) -> list:
 def geography(input: _Union[str, list], dataType: str) -> list:
     """
     Extract geographical information from unstructured text such as streets, cities or countries.
+    Requires WrangleWorks Account.
 
     e.g. '1100 Congress Ave, Austin, TX 78701, United States' -> '1100 Congress Ave'
 
@@ -157,6 +171,7 @@ def geography(input: _Union[str, list], dataType: str) -> list:
 def html(input: _Union[str, list], dataType: str) -> list:
     """
     Extract specific html elements from strings containing html.
+    Requires WrangleWorks Account.
 
     :param input: A string or list of strings with addresses to search for information.
     :param dataType: The type of information to return. 'text' or 'links'
@@ -181,6 +196,7 @@ def html(input: _Union[str, list], dataType: str) -> list:
 def properties(input: _Union[str, list], type: str = None) -> _Union[dict, list]:
     """
     Extract categorical properties from unstructured text such as colours or materials.
+    Requires WrangleWorks Account.
 
     >>> wrangles.extract.properties('The Green Mile')
     {'Colours': ['Green']}
@@ -208,38 +224,47 @@ def properties(input: _Union[str, list], type: str = None) -> _Union[dict, list]
 
 
 # SUPER MARIO
-def remove_words(input, to_remove):
+def remove_words(input, to_remove, tokenize_to_remove, ignore_case):
     """
     Remove all the elements that occur in one list from another.
     
     :param input: both input and to_remove can be a string or a list or multiple lists. Lowered for precision
     :param output: a string of remaining words
     """
-        
-    # Input is string
+    
+    # Tokenize to_remove values
+    if tokenize_to_remove == True:
+        to_remove = [tokenize(to_remove[x]) for x in range(len(to_remove))]
+            
+    # If Input is a string
     if isinstance(input[0], str):
-        input_lower = [x.lower() for x in input]
-        input = [[x.split()] for x in input_lower]
+        input = [x.split() for x in input]
+    
+    if ignore_case == True:
+        """
+        Takes inputs and converts to lower
+        """
+    
         results = []
         for item in range(len(input)):
             temp = []
             to_remove_lower = [item.lower() for sublist in to_remove[item] for item in sublist]
-            temp = filter(None, [x.title() for x in input[item][0] if x not in to_remove_lower])
+            input_lower = [x.lower() for x  in input[item]]
+            temp = filter(None, [x.title() for x in input_lower if x not in to_remove_lower])
+            results.append(' '.join(temp))
+            
+    else:
+        """
+        Takes inputs as is (raw)
+        """
+        results = []
+        for item in range(len(input)):
+            temp = []
+            to_remove_lower = [item for sublist in to_remove[item] for item in sublist]
+            input_lower = [x for x  in input[item]]
+            temp = filter(None, [x for x in input_lower if x not in to_remove_lower])
             results.append(' '.join(temp))
-        
-        return results
-        
-    
-    # Input is list
-    results = []
-    for item in range(len(input)):
-        temp = []
-        to_remove_lower = [item.lower() for sublist in to_remove[item] for item in sublist]
-        input_lower = [x.lower() for x  in input[item]]
-        temp = filter(None, [x.title() for x in input_lower if x not in to_remove_lower])
-        results.append(' '.join(temp))
 
-        
     return results
     
     
\ No newline at end of file
