@@ -11,13 +11,16 @@ def join_list(input_list, join_char):
 
 
 def concatenate(data_list, concat_char):
+    """
+    Concatenate a list of columns
+    """
     results = []
     for row in data_list:
         results.append(concat_char.join(row))            
     return results
     
 # Super Mario Function
-def split_re(input_list, split_char):
+def split_re(input_list, split_char, output):
     # split char is a list of characters -> Joining them
     if isinstance(split_char, list):
         split_char = '|'.join(split_char)
@@ -25,11 +28,18 @@ def split_re(input_list, split_char):
     results = [re.split(split_char, x) for x in input_list]
     return results
     
-def split(input_list, split_char, pad=False):
+def split(input_list, split_char, output, pad=False):
     if pad:
         # Pad to be as long as the longest result
         max_len = max([len(x.split(split_char)) for x in input_list])
         results = [x.split(split_char) + [''] * (max_len - len(x.split(split_char))) for x in input_list]
+        
+        # trimming list to appropriate output columns number
+        if len(output) <= max_len and isinstance(output, list):
+            results = [x[:len(output)] for x in results]
+        # if more columns than number of splits, then add '' in extra columns
+        elif len(output) >= max_len and isinstance(output, list):
+            results = [x + [''] * (len(output) - len(x)) for x in results] 
     else:
         results = [x.split(split_char) for x in input_list]
     return results
@@ -54,6 +64,9 @@ def coalesce(input_list: list) -> list:
 
 
 def price_breaks(df_input, header_cat, header_val): # pragma: no cover
+    """
+    Rearrange price breaks
+    """
     output = []
     headers = []
     i = 1
