@@ -1,4 +1,7 @@
 def highest_confidence(data_list):
+    """
+    Select the option with the highest confidence from multiple columns
+    """
     results = []
     for row in data_list:
         highest_confidence = 0
@@ -14,6 +17,9 @@ def highest_confidence(data_list):
     return results
 
 def confidence_threshold(list_1, list_2, threshold):
+    """
+    Select the first option if it exceeds a given threshold, else the second option.
+    """
     results = []
     
     for cell_1, cell_2 in zip(list_1, list_2):
@@ -35,14 +41,20 @@ def confidence_threshold(list_1, list_2, threshold):
     return results
 
 def list_element(input, n):
-    def check_if_possible(a, element):
+    """
+    Select a numbered element of a list (zero indexed).
+    """
+    def check_if_possible(element, index):
         try:
-            a[element]
-            return a[element]
+            return element[index]
         except IndexError:
             return ''
-            
-    return [check_if_possible(row, n) for row in input]
+        
+    return [check_if_possible(row, n) if isinstance(row, list) else '' for row in input]
 
 def dict_element(input, key):
-    return [row.get(key, '') for row in input]
\ No newline at end of file
+    """
+    Select a named element of a dictionary
+    """
+    return [row.get(key, '') if isinstance(row, dict) else '' for row in input]
+        
\ No newline at end of file
